import 'package:ai_app/repositories/ai_repository.dart';
import 'package:ai_app/services/http_service.dart';
import 'package:flutter/material.dart';
import 'package:ai_app/screens/introduction_screen.dart';
import 'package:ai_app/globals/global_variables.dart' as global_variables;

class AiApp extends StatefulWidget {
  const AiApp({
    required this.token,
    required this.firstName,
    required this.lastName,
    Key? key,
  }) : super(key: key);
  final String token;
  final String firstName;
  final String lastName;

  @override
  _AiAppState createState() => _AiAppState();
}

class _AiAppState extends State<AiApp> {
  late AiRepository aiRepository;

  late HttpService httpService;

  @override
  void initState() {
    httpService = HttpService();
    aiRepository = AiRepository(httpService);
    global_variables.currentUser.firstName = widget.firstName;
    global_variables.currentUser.lastName = widget.lastName;
    _getCurrentUser();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return IntroductionScreen();
  }

  Future<void> _getCurrentUser() async {
    await aiRepository
        .authenticateUser(
          token: this.widget.token,
          firstName: this.widget.firstName,
          lastName: this.widget.lastName,
        )
        .then((user) async => {
              setState(
                () {
                  global_variables.currentUser = user;
                },
              ),
            });
  }
}
