import 'dart:async';
import 'dart:convert';

import 'package:ai_app/models/answer_model.dart';
import 'package:ai_app/models/question_model.dart';
import 'package:ai_app/models/user_model.dart';
import 'package:ai_app/services/http_service.dart';

class AiRepository {
  final HttpService _httpService;

  // ignore: unnecessary_null_comparison
  AiRepository(this._httpService) : assert(_httpService != null);

  Future<UserModel> authenticateUser({
    token,
    firstName,
    lastName,
  }) {
    return _httpService
        .authenticateUser(
      token: token,
      firstName: firstName,
      lastName: lastName,
    )
        .then((data) async {
      if (data.statusCode >= 200 && data.statusCode < 300) {
        var res = json.decode(data.body);
        UserModel userData = UserModel.fromJson(res);

        return userData;
      }
      return UserModel();
    });
  }

  Future<List<QuestionModel>> getAllQuestions() {
    return _httpService.getAllQuestions().then((data) async {
      if (data.statusCode == 200) {
        List<dynamic> tags = json.decode(data.body);
        List<QuestionModel> questionsList =
            tags.map((i) => QuestionModel.fromJson(i)).toList();
        return questionsList;
      } else {
        return List<QuestionModel>.from([]);
      }
    });
  }

  Future<AnswerModel?> createAnswer(
      {required String answer, required int? user, int? question}) {
    return _httpService
        .createAnswer(answer: answer, user: user, question: question)
        .then(
      (data) async {
        if (data.statusCode >= 200 && data.statusCode < 300) {
          var result = json.decode(data.body);
          AnswerModel answerData = AnswerModel.fromJson(result[0]);
          return answerData;
        }
        return null;
      },
    );
  }
}
