import 'dart:async';

import 'package:ai_app/globals/global_functions.dart';
// ignore: unused_import
import 'package:ai_app/widgets/ai_outlined_button.dart';
import 'package:camera/camera.dart';
// ignore: unused_import
import 'package:cloudinary_public/cloudinary_public.dart';
import 'package:flutter/material.dart';
// ignore: unused_import
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:ai_app/globals/global_colors.dart';

import 'package:ai_app/globals/global_variables.dart' as global_variables;
import 'package:ai_app/repositories/ai_repository.dart';
import 'package:ai_app/services/http_service.dart';

class InterviewScreen extends StatefulWidget {
  const InterviewScreen({
    Key? key,
    required this.questionNum,
  }) : super(key: key);

  final int questionNum;
  @override
  _InterviewScreenState createState() {
    return _InterviewScreenState();
  }
}

void logError(String code, String? message) {
  if (message != null) {
    print('Error: $code\nError Message: $message');
  } else {
    print('Error: $code');
  }
}

class _InterviewScreenState extends State<InterviewScreen>
    with WidgetsBindingObserver, TickerProviderStateMixin {
  CameraController? controller;
  XFile? videoFile;
  VoidCallback? videoPlayerListener;
  bool isBigDoll = false;
  bool isStopButtonPressed = false;
  late HttpService httpService;
  late AiRepository aiRepository;
  late Timer _timer;
  int seconds = 0;
  int minutes = 0;
  int hours = 0;
  String secondsString = "00";
  String minutesString = "00";
  String hoursString = "00";
  final _cameraWidgetKey = GlobalKey();
  var _cameraWidgetSize;

  bool enableAudio = true;
  @override
  void initState() {
    httpService = HttpService();
    aiRepository = AiRepository(httpService);
    super.initState();
    WidgetsBinding.instance?.addObserver(this);
    _showCamera();
  }

  @override
  void dispose() {
    WidgetsBinding.instance?.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isInitialized) {
      return;
    }

    if (state == AppLifecycleState.inactive) {
      cameraController.dispose();
    } else if (state == AppLifecycleState.resumed) {
      onNewCameraSelected(cameraController.description);
    }
  }

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(
        global_variables.screenWidth,
        global_variables.screenHeight,
      ),
      builder: () => Scaffold(
        key: _scaffoldKey,
        body: _buildContent(),
      ),
    );
  }

  _buildContent() {
    return Column(
      children: <Widget>[
        Center(
          child: _cameraPreviewWidget(),
        ),
      ],
    );
  }

  _cameraPreviewWidget() {
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isInitialized) {
      return Text('');
    } else {
      return _buildMain();
    }
  }

  _buildMain() {
    return isBigDoll == false
        ? Stack(
            children: [
              _buildCamera(),
              _buildDoll(),
              _captureControlRowWidget(),
              _buildAutorenewButton(),
            ],
          )
        : Stack(
            children: [
              _buildDoll(),
              _buildCamera(),
              _captureControlRowWidget(),
              _buildAutorenewButton(),
            ],
          );
  }

  _buildCamera() {
    return Padding(
      padding: isBigDoll == false
          ? const EdgeInsets.all(0)
          : EdgeInsets.only(
              top: GlobalFunctions().setHeight(context, 91),
              left: GlobalFunctions().setWidth(context, 254),
            ),
      child: Container(
        height: isBigDoll == false
            ? null
            : GlobalFunctions().setHeight(context, 123),
        width: isBigDoll == false
            ? MediaQuery.of(context).size.width
            : GlobalFunctions().setWidth(context, 97),
        child: Container(
          decoration: isBigDoll == true
              ? BoxDecoration(
                  borderRadius: BorderRadius.circular(8),
                  border: Border.all(
                    color: GlobalColors.aiGrey,
                    width: 1,
                  ),
                )
              : null,
          child: ClipRRect(
            key: _cameraWidgetKey,
            borderRadius: isBigDoll == true
                ? BorderRadius.circular(8)
                : BorderRadius.circular(0),
            child: CameraPreview(
              controller!,
            ),
          ),
        ),
      ),
    );
  }

  Widget _captureControlRowWidget() {
    final CameraController? cameraController = controller;

    return Positioned(
      bottom: GlobalFunctions().setHeight(context, 22),
      left: GlobalFunctions().setWidth(context, 62),
      child: Column(
        children: [
          Container(
            child: Column(
              children: [
                Container(
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(12),
                    color: Color(0xFF6C7176).withOpacity(0.7),
                  ),
                  height: GlobalFunctions().setHeight(context, 70),
                  width: GlobalFunctions().setWidth(context, 250),
                  child: Padding(
                    padding: EdgeInsets.only(
                      top: GlobalFunctions().setHeight(context, 7),
                      bottom: GlobalFunctions().setHeight(context, 7),
                    ),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      mainAxisSize: MainAxisSize.max,
                      children: <Widget>[
                        Container(
                          height: GlobalFunctions().setHeight(context, 56),
                          width: GlobalFunctions().setWidth(context, 56),
                          decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: GlobalColors.aiIconButtonGrey,
                          ),
                          child: IconButton(
                            icon: Icon(
                              enableAudio ? Icons.volume_up : Icons.volume_mute,
                              size: GlobalFunctions().setWidth(context, 22),
                            ),
                            color: Colors.white,
                            onPressed: controller != null
                                ? onAudioModeButtonPressed
                                : null,
                          ),
                        ),
                        Container(
                          height: GlobalFunctions().setHeight(context, 56),
                          width: GlobalFunctions().setWidth(context, 56),
                          decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: GlobalColors.aiRed,
                            border: Border.all(
                              color: Colors.white,
                              width: GlobalFunctions().setWidth(context, 3),
                            ),
                          ),
                          child: IconButton(
                            icon: cameraController != null &&
                                    cameraController.value.isRecordingPaused
                                ? Icon(Icons.play_arrow)
                                : Icon(
                                    Icons.stop,
                                    size:
                                        GlobalFunctions().setWidth(context, 22),
                                  ),
                            color: Colors.white,
                            onPressed: cameraController != null &&
                                    cameraController.value.isInitialized &&
                                    cameraController.value.isRecordingVideo
                                ? (cameraController.value.isRecordingPaused)
                                    ? onResumeButtonPressed
                                    : onPauseButtonPressed
                                : () {
                                    GlobalFunctions()
                                        .showInSnackBar(_scaffoldKey, "null");
                                  },
                          ),
                        ),
                        Container(
                          height: GlobalFunctions().setHeight(context, 56),
                          width: GlobalFunctions().setWidth(context, 56),
                          decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: GlobalColors.aiIconButtonGrey,
                          ),
                          child: IconButton(
                            icon: Icon(
                              Icons.arrow_forward,
                              size: GlobalFunctions().setWidth(context, 22),
                            ),
                            color: Colors.white,
                            onPressed: _stopVideo,
                          ),
                        )
                      ],
                    ),
                  ),
                ),
                _buildTimer(),
              ],
            ),
          ),
        ],
      ),
    );
  }

  void onAudioModeButtonPressed() {
    setState(() {
      enableAudio = !enableAudio;
    });
  }

  _showCamera() {
    final playCamera = (CameraDescription? description) {
      if (description == null) {
        return;
      }

      onNewCameraSelected(description);
    };

    if (global_variables.cameras.isEmpty) {
      GlobalFunctions().showInSnackBar(_scaffoldKey, 'No camera found');
    } else {
      for (CameraDescription cameraDescription in global_variables.cameras) {
        if (cameraDescription.lensDirection == CameraLensDirection.front) {
          playCamera(cameraDescription);
          break;
        }
      }
    }
  }

  void onNewCameraSelected(CameraDescription cameraDescription) async {
    if (controller != null) {
      await controller!.dispose();
    }
    final CameraController cameraController = CameraController(
      cameraDescription,
      ResolutionPreset.medium,
      enableAudio: enableAudio,
      imageFormatGroup: ImageFormatGroup.jpeg,
    );
    controller = cameraController;

    cameraController.addListener(() {
      if (mounted) setState(() {});
      if (cameraController.value.hasError) {
        GlobalFunctions().showInSnackBar(_scaffoldKey,
            'Camera error ${cameraController.value.errorDescription}');
      }
    });

    try {
      await cameraController.initialize();
      await Future.wait([cameraController.getMinExposureOffset()]);
    } on CameraException catch (e) {
      _showCameraException(e);
    }

    if (mounted) {
      setState(() {});
    }

    if (cameraController.value.isInitialized &&
        !cameraController.value.isRecordingVideo) {
      onVideoRecordButtonPressed();
    }
  }

  void onVideoRecordButtonPressed() {
    _startTimer();
    startVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  void onStopButtonPressed() {
    _stopTimer();
    setState(() {
      isStopButtonPressed = true;
    });
  }

  void onPauseButtonPressed() {
    pauseVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  void onResumeButtonPressed() {
    resumeVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  Future<void> startVideoRecording() async {
    final CameraController? cameraController = controller;
    if (cameraController == null || !cameraController.value.isInitialized) {
      GlobalFunctions()
          .showInSnackBar(_scaffoldKey, 'Error: select a camera first.');
      return;
    }

    if (cameraController.value.isRecordingVideo) {
      return;
    }

    try {
      await cameraController.startVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return;
    }
  }

  Future<XFile?> stopVideoRecording() async {
    _stopTimer();
    final CameraController? cameraController = controller;
    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      return cameraController.stopVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  Future<void> pauseVideoRecording() async {
    _timer.cancel();
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      await cameraController.pauseVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<void> resumeVideoRecording() async {
    _startTimer();
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      await cameraController.resumeVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  void _showCameraException(CameraException e) {
    logError(e.code, e.description);
    GlobalFunctions()
        .showInSnackBar(_scaffoldKey, 'Error: ${e.code}\n${e.description}');
  }

  _buildDoll() {
    return Padding(
      padding: isBigDoll == true
          ? const EdgeInsets.all(0)
          : EdgeInsets.only(
              top: GlobalFunctions().setHeight(context, 91),
              left: GlobalFunctions().setWidth(context, 254),
            ),
      child: Container(
        decoration: isBigDoll == false
            ? BoxDecoration(
                borderRadius: BorderRadius.circular(8),
                color: Colors.white,
                border: Border.all(
                  color: GlobalColors.aiGrey,
                  width: 1,
                ),
              )
            : BoxDecoration(
                color: GlobalColors.aiDollBackground,
              ),
        height: isBigDoll == true
            ? _cameraWidgetSize != null
                ? _cameraWidgetSize.height
                : null
            : GlobalFunctions().setHeight(context, 123),
        width: isBigDoll == true
            ? MediaQuery.of(context).size.width
            : GlobalFunctions().setWidth(context, 97),
        child: Padding(
          padding: isBigDoll == true
              ? EdgeInsets.only(
                  right: GlobalFunctions().setWidth(context, 125),
                )
              : EdgeInsets.all(0),
          child: Image.network(
            'https://res.cloudinary.com/feed-phonedo/image/upload/v1623825359/doll_b3aubv.png',
            fit: BoxFit.fitHeight,
          ),
        ),
      ),
    );
  }

  _buildAutorenewButton() {
    return Padding(
      padding: EdgeInsets.only(
        left: GlobalFunctions().setWidth(context, 321),
        top: GlobalFunctions().setHeight(context, 197),
      ),
      child: Container(
        height: GlobalFunctions().setHeight(context, 32),
        width: GlobalFunctions().setWidth(context, 32),
        child: RawMaterialButton(
          onPressed: () {
            _getSize();
            setState(() {
              isBigDoll = !isBigDoll;
            });
          },
          fillColor: Colors.black,
          shape: CircleBorder(),
          child: Icon(
            Icons.autorenew,
            color: Colors.white,
          ),
        ),
      ),
    );
  }

  void _stopVideo() async {
    stopVideoRecording().then((file) async {
      if (mounted)
        setState(() {
          Navigator.pop(context);
        });
      if (file != null) {
        videoFile = file;
        final cloudinary =
            CloudinaryPublic('ruthphonedo', 'hpwimk0n', cache: false);
        var videoPath = file.path;

        try {
          CloudinaryResponse response = await cloudinary.uploadFile(
            CloudinaryFile.fromFile(videoPath,
                resourceType: CloudinaryResourceType.Video),
          );

          await aiRepository.createAnswer(
            answer: response.secureUrl,
            user: global_variables.currentUser.id,
            question: widget.questionNum,
          );
        } on CloudinaryException catch (e) {
          print(e.message);
          print(e.request);
        }
      }
    });
  }

  _buildTimer() {
    return Padding(
      padding: EdgeInsets.only(
        top: GlobalFunctions().setHeight(context, 26),
      ),
      child: Center(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(16),
            color: GlobalColors.aiRed,
          ),
          height: GlobalFunctions().setHeight(context, 26),
          width: GlobalFunctions().setWidth(context, 76),
          child: Center(
            child: Text(
              hoursString.toString() +
                  ":" +
                  minutesString.toString() +
                  ":" +
                  secondsString.toString(),
              style: TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
      ),
    );
  }

  void _startTimer() {
    const oneSec = const Duration(seconds: 1);
    _timer = new Timer.periodic(
      oneSec,
      (Timer timer) => setState(
        () {
          if (seconds < 0) {
            timer.cancel();
          } else {
            seconds = seconds + 1;
            secondsString =
                seconds > 9 ? seconds.toString() : "0" + seconds.toString();
            if (seconds > 59) {
              minutes += 1;

              seconds = 0;
              secondsString = "00";
              minutesString =
                  minutes > 9 ? minutes.toString() : "0" + minutes.toString();
              if (minutes > 59) {
                hours += 1;

                minutes = 0;
                minutesString = "00";
                hoursString =
                    hours > 9 ? hours.toString() : "0" + hours.toString();
              }
            }
          }
        },
      ),
    );
  }

  _stopTimer() {
    setState(() {
      seconds = 0;
      minutes = 0;
      hours = 0;
      secondsString = "00";
      minutesString = "00";
      hoursString = "00";
    });
    _timer.cancel();
  }

  _getSize() {
    setState(() {
      _cameraWidgetSize = _cameraWidgetKey.currentContext?.size;
    });
  }
}
