import 'dart:async';

import 'package:ai_app/globals/global_functions.dart';
import 'package:ai_app/models/question_model.dart';
import 'package:ai_app/repositories/ai_repository.dart';
import 'package:ai_app/services/http_service.dart';
// ignore: implementation_imports
import 'package:audioplayers/src/notifications/player_control_command.dart';
import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:ai_app/globals/global_colors.dart';
import 'package:ai_app/globals/global_variables.dart' as global_variables;
import 'package:ai_app/screens/interview_screen.dart';
import 'package:ai_app/widgets/ai_outlined_button.dart';
import 'package:audioplayers/audioplayers.dart';

class IntroductionScreen extends StatefulWidget {
  @override
  _IntroductionScreenState createState() => _IntroductionScreenState();

  static const timeout = Duration(seconds: 10);
  static const ms = Duration(milliseconds: 1);
}

class _IntroductionScreenState extends State<IntroductionScreen> {
  int screenNum = 1;

  int questionNum = 0;

  late String message1;

  late String message2;

  late String message3;

  late String message4;

  late String message5;

  late String message6;

  late List<QuestionModel> questionsList;

  late HttpService httpService;

  late AiRepository aiRepository;

  late List<String> questionsListString = [];

  late List<String> questionsListAudioUrl = [];

  late AudioPlayer _audioPlayer;
  Duration? _duration;
  Duration? _position;
  // ignore: unused_field
  PlayerState _playerState = PlayerState.STOPPED;

  late PlayerMode mode = PlayerMode.MEDIA_PLAYER;

  // ignore: unused_field
  PlayingRoute _playingRouteState = PlayingRoute.SPEAKERS;

  StreamSubscription? _positionSubscription;

  StreamSubscription? _durationSubscription;

  StreamSubscription? _playerCompleteSubscription;

  StreamSubscription? _playerErrorSubscription;

  StreamSubscription<PlayerControlCommand>? _playerControlCommandSubscription;

  // ignore: unused_field
  PlayerState? _audioPlayerState;

  List<String> messages = [];

  void initState() {
    message1 =
        "Welcome to the Phone.do Interview. \nMy name is Jenny and I will ask you \nquestions during all interview. \n ";
    message2 =
        "Fill free to asked me anything in case \nyou did not understand me or if you \nhave any question during the \ninterview";
    message3 =
        "during the interview, a chat room will \nbe opened, so you can also send me \ntext messages and I will answer you \nback. ";

    message4 = "Ok, " +
        global_variables.currentUser.firstName.toString() +
        ", I’m done introducing myself. \nIn about 3 seconds a “Record” button \nwill appear, So feel free to click on it \nwhen you ready to start.";

    message5 =
        "Unfortunately, you stopped the \ninterview in the middle so I can not \nallow you to continue using our \nservice, thanks, and good luck!";
    message6 = "Thank you " +
        global_variables.currentUser.firstName.toString() +
        ", for participating in \nour interview. We will go over the data \nand get back to you soon.";
    messages = [message1, message2, message3, message4, message5, message6];

    startTimeout(2000);
    httpService = HttpService();
    aiRepository = AiRepository(httpService);
    _getAllQuestions();
    super.initState();
  }

  @override
  void dispose() {
    _audioPlayer.dispose();
    _durationSubscription?.cancel();
    _positionSubscription?.cancel();
    _playerCompleteSubscription?.cancel();
    _playerErrorSubscription?.cancel();
    _playerControlCommandSubscription?.cancel();
    super.dispose();
  }

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(
        global_variables.screenWidth,
        global_variables.screenHeight,
      ),
      builder: () => Scaffold(
        key: _scaffoldKey,
        body: _buildContent(),
      ),
    );
  }

  _buildContent() {
    return Column(
      children: [
        Expanded(
          child: Column(
            children: [
              screenNum == 5
                  ? Text("")
                  : _buildTitle(
                      text: screenNum == 8
                          ? "Thank You!"
                          : screenNum == 9
                              ? "Question no. " + questionNum.toString()
                              : "Hello, " +
                                  global_variables.currentUser.firstName
                                      .toString() +
                                  " " +
                                  global_variables.currentUser.lastName
                                      .toString()),
              _buildMain(),
            ],
          ),
        ),
        _buildDoll(),
      ],
    );
  }

  _buildTitle({required String text}) {
    return Padding(
      padding: EdgeInsets.only(
        top: GlobalFunctions().setHeight(context, 105),
      ),
      child: Center(
        child: Text(
          text,
          style: TextStyle(
            fontSize: ScreenUtil().setSp(24),
            color: GlobalColors.aiBlue,
          ),
        ),
      ),
    );
  }

  _buildMain() {
    return Padding(
      padding: EdgeInsets.only(
        top: GlobalFunctions().setHeight(context, 39),
      ),
      child: Center(
        child: screenNum == 5
            ? _buildRecordButton()
            : screenNum == 7
                ? _buildDone()
                : screenNum == 8
                    ? _buildStop()
                    : _buildMessageItem(),
      ),
    );
  }

  _buildMessageItem() {
    return Column(
      children: [
        Container(
          width: GlobalFunctions().setWidth(context, 264),
          decoration: BoxDecoration(
            color: GlobalColors.aiLightGrey,
            borderRadius: BorderRadius.circular(10),
          ),
          child: Padding(
            padding: EdgeInsets.only(
              top: GlobalFunctions().setHeight(context, 14),
              left: GlobalFunctions().setWidth(context, 13),
              bottom: GlobalFunctions().setHeight(context, 18),
            ),
            child: Text(
              screenNum < 5
                  ? messages[screenNum - 1]
                  : screenNum == 6
                      ? message5
                      : screenNum == 8
                          ? message6
                          : screenNum == 9
                              ? questionsListString.length > 0
                                  ? questionsListString[questionNum - 1]
                                  : "no internet"
                              : "",
              style: TextStyle(
                fontSize: ScreenUtil().setSp(14),
                height: screenNum == 8
                    ? ScreenUtil().setSp(1.4)
                    : ScreenUtil().setSp(1.7),
                color: GlobalColors.aiBlue,
              ),
            ),
          ),
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 5),
            left: GlobalFunctions().setWidth(context, 258.76),
          ),
          child: Container(
            decoration: new BoxDecoration(
              color: GlobalColors.aiLightGrey,
              shape: BoxShape.circle,
            ),
            height: GlobalFunctions().setHeight(context, 11),
            width: GlobalFunctions().setWidth(context, 11),
          ),
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 5),
            left: GlobalFunctions().setWidth(context, 247),
          ),
          child: Container(
            decoration: new BoxDecoration(
              color: GlobalColors.aiLightGrey,
              shape: BoxShape.circle,
            ),
            height: GlobalFunctions().setHeight(context, 8.25),
            width: GlobalFunctions().setWidth(context, 8.25),
          ),
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 5),
            left: GlobalFunctions().setWidth(context, 236),
          ),
          child: Container(
            decoration: new BoxDecoration(
              color: GlobalColors.aiLightGrey,
              shape: BoxShape.circle,
            ),
            height: GlobalFunctions().setHeight(context, 5.5),
            width: GlobalFunctions().setWidth(context, 5.5),
          ),
        ),
      ],
    );
  }

  _buildRecordButton() {
    return Padding(
      padding: EdgeInsets.only(top: GlobalFunctions().setHeight(context, 109)),
      child: Column(
        children: [
          Container(
            width: GlobalFunctions().setWidth(context, 80),
            height: GlobalFunctions().setHeight(context, 80),
            decoration: BoxDecoration(
              shape: BoxShape.circle,
              boxShadow: [
                BoxShadow(
                  color: Colors.grey.withOpacity(0.5),
                  spreadRadius: 4,
                  blurRadius: 10,
                  offset: Offset(0, 0),
                )
              ],
            ),
            child: RawMaterialButton(
              onPressed: () {
                _onRecordButtonPressed();
              },
              fillColor: Colors.white,
              child: Padding(
                padding: EdgeInsets.all(
                  GlobalFunctions().setHeight(context, 26),
                ),
                child: Container(
                  decoration: BoxDecoration(
                    color: Colors.red,
                    shape: BoxShape.circle,
                  ),
                ),
              ),
              shape: CircleBorder(),
            ),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: GlobalFunctions().setHeight(context, 26),
            ),
            child: Text(
              "Start recording...",
              style: TextStyle(
                fontSize: ScreenUtil().setSp(20),
                color: GlobalColors.aiBlue,
              ),
            ),
          ),
        ],
      ),
    );
  }

  _buildDoll() {
    return Expanded(
      child: Container(
        child: Image.network(
          'https://res.cloudinary.com/feed-phonedo/image/upload/v1623825359/doll_b3aubv.png',
        ),
      ),
    );
  }

  _buildStop() {
    return Column(
      children: [
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 24),
          ),
          child: AiOutlinedButton(
            onPressed: () {
              SystemNavigator.pop();
            },
            text: "Logout",
            isGreenButton: false,
            isBlueFont: true,
          ),
        ),
      ],
    );
  }

  _buildEnd() {
    return Column(
      children: [
        AiOutlinedButton(
          onPressed: () {
            setState(() {
              screenNum = 0;
              questionNum = 0;
            });
            handleTimeout();
          },
          text: "Start a new interview",
          isGreenButton: true,
          isBlueFont: false,
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 24),
          ),
          child: AiOutlinedButton(
            onPressed: () {
              setState(() {
                screenNum = 0;
                questionNum = 0;
              });
              handleTimeout();
            },
            text: "Logout",
            isGreenButton: false,
            isBlueFont: true,
          ),
        ),
      ],
    );
  }

  _buildDone() {
    return Column(
      children: [
        AiOutlinedButton(
          onPressed: () {
            setState(() {
              screenNum = 0;
              questionNum = 0;
            });
            handleTimeout();
          },
          text: "Done",
          isGreenButton: true,
          isBlueFont: false,
        ),
        Padding(
          padding: EdgeInsets.only(
            top: GlobalFunctions().setHeight(context, 41),
          ),
          child: _buildMessageItem(),
        ),
      ],
    );
  }

  Timer startTimeout([int? milliseconds]) {
    var duration = milliseconds == null
        ? IntroductionScreen.timeout
        : IntroductionScreen.ms * milliseconds;
    return Timer(duration, handleTimeout);
  }

  void handleTimeout() {
    if (screenNum < 5) {
      setState(() {
        screenNum++;
      });
      startTimeout(2000);
    }
  }

  _onRecordButtonPressed() {
    if (questionsListString.length > 0) {
      if (questionNum < questionsListString.length) {
        setState(() {
          screenNum = 9;
          questionNum++;
        });

        _initAudioPlayer();
        _playAudio(url: questionsListAudioUrl[questionNum - 1]);
      } else {
        setState(() {
          screenNum = 8;
        });
      }
    } else {
      GlobalFunctions().showInSnackBar(_scaffoldKey, "no internet");
    }
  }

  _navigateToInterview() async {
    try {
      WidgetsFlutterBinding.ensureInitialized();
      global_variables.cameras = await availableCameras();
    } on CameraException catch (e) {
      logError(
        e.code,
        e.description,
      );
    }

    Navigator.push(
      context,
      MaterialPageRoute(
        builder: (context) => InterviewScreen(questionNum: questionNum),
      ),
    ).then((value) => _onRecordButtonPressed());
  }

  Future<void> _getAllQuestions() async {
    await aiRepository.getAllQuestions().then((value) async => {
          setState(() {
            questionsList = value;
            for (QuestionModel q in questionsList) {
              questionsListString.add(q.question);
              questionsListAudioUrl.add(q.audioUrl);
            }
          })
        });
  }

  Future<int> _playAudio({required String url}) async {
    final playPosition = (_position != null &&
            _duration != null &&
            _position!.inMilliseconds > 0 &&
            _position!.inMilliseconds < _duration!.inMilliseconds)
        ? _position
        : null;
    final result = await _audioPlayer.play(url, position: playPosition);
    if (result == 1) {
      setState(() => _playerState = PlayerState.PLAYING);
    }

    _audioPlayer.setPlaybackRate();

    return result;
  }

  void _initAudioPlayer() {
    _audioPlayer = AudioPlayer(mode: mode);

    _durationSubscription = _audioPlayer.onDurationChanged.listen((duration) {
      setState(() => _duration = duration);

      if (Theme.of(context).platform == TargetPlatform.iOS) {
        _audioPlayer.notificationService.startHeadlessService();

        _audioPlayer.notificationService.setNotification(
          title: 'App Name',
          artist: 'Artist or blank',
          albumTitle: 'Name or blank',
          imageUrl: 'Image URL or blank',
          forwardSkipInterval: const Duration(seconds: 30),
          backwardSkipInterval: const Duration(seconds: 30),
          duration: duration,
          enableNextTrackButton: true,
          enablePreviousTrackButton: true,
        );
      }
    });

    _positionSubscription =
        _audioPlayer.onAudioPositionChanged.listen((p) => setState(() {
              _position = p;
            }));

    _playerCompleteSubscription =
        _audioPlayer.onPlayerCompletion.listen((event) {
      _onComplete();
      setState(() {
        _position = _duration;
      });
    });

    _playerErrorSubscription = _audioPlayer.onPlayerError.listen((msg) {
      print('audioPlayer error : $msg');
      setState(() {
        _playerState = PlayerState.STOPPED;
        _duration = const Duration();
        _position = const Duration();
      });
    });

    _playerControlCommandSubscription =
        _audioPlayer.notificationService.onPlayerCommand.listen((command) {
      print('command: $command');
    });

    _audioPlayer.onPlayerStateChanged.listen((state) {
      if (mounted) {
        setState(() {
          _audioPlayerState = state;
        });
      }
    });

    _audioPlayer.onNotificationPlayerStateChanged.listen((state) {
      if (mounted) {
        setState(() => _audioPlayerState = state);
      }
    });

    _playingRouteState = PlayingRoute.SPEAKERS;
  }

  void _onComplete() {
    setState(() {
      _playerState = PlayerState.STOPPED;
    });
    _navigateToInterview();
  }
}
