import 'dart:convert';
import 'dart:developer';
import 'package:http/http.dart' as http;
import 'package:http/http.dart';
import 'package:ai_app/exeptions/auth_exception.dart';

class HttpService {
  final String apiURL = 'http://10.0.2.2:3000';

  Future<Response> get(url, {Map<String, String>? headers}) async {
    final response = await http.get(url, headers: headers);

    if (response.statusCode == 401) {
      if (json.decode(response.body)['message'] == 'Expired JWT Token') {
        throw ExpiredTokenException(response.reasonPhrase);
      } else {
        throw AuthenticateException(response.reasonPhrase);
      }
    } else {
      return response;
    }
  }

  Future<Response> post(
      {required url,
      Map<String, String>? headers,
      body,
      Encoding? encoding}) async {
    final response =
        await http.post(url, headers: headers, body: body, encoding: encoding);
    if (response.statusCode == 401 &&
        json.decode(response.body)['message'] == 'Expired JWT Token') {
      throw ExpiredTokenException(response.reasonPhrase);
    } else {
      return response;
    }
  }

  Future<http.Response> authenticateUser({
    required token,
    required firstName,
    required lastName,
  }) {
    return get(
        Uri.parse(
            '$apiURL/user/authenticate?token=$token&firstName=$firstName&lastName=$lastName'),
        headers: {});
  }

  Future<http.Response> getAllQuestions() {
    return get(
      Uri.parse('$apiURL/question/all'),
    );
  }

  Future<http.Response> createAnswer(
      {required String answer, required int? user, required int? question}) {
    var data =
        json.encode({"answer": answer, "user": user, "question": question});
    log(data);

    return post(
      url: Uri.parse('$apiURL/answer/create'),
      headers: {
        "content-type": 'application/json',
      },
      body: data,
    );
  }
}
